// #ifdef SMART_POC_MODULE_DWD_WX
#include "dwdwx.h"

DWDWXClient::DWDWXClient(WiFiClient *client)
{
    _client = client;
}

void DWDWXClient::begin(int intervalMinutes, float latitude, float longitude)
{
    // this->message = 
    this->url = "http://api.open-meteo.com/v1/forecast?latitude=";
    this->url.concat(latitude);
    this->url.concat("&longitude=");
    this->url.concat(longitude);
    this->url.concat("&hourly=temperature_2m&models=icon_seamless&current=temperature_2m,relative_humidity_2m,wind_direction_10m,wind_speed_10m,wind_gusts_10m,pressure_msl,cloud_cover");

    this->checkInterval = 60e3 * intervalMinutes;
    Serial.printf("[DWDWX]: check Interval : %dms\n", this->checkInterval);
    this->check();
}
void DWDWXClient::loop()
{
    if (millis() - this->lastCheck >= this->checkInterval)
    {
        // retry once
        if (!this->check())
            this->check();
        this->lastCheck = millis();
    }
}
bool DWDWXClient::isDirty()
{
    if (this->_dirty)
    {
        this->_dirty = false;
        return true;
    }
    return false;
}
bool DWDWXClient::check()
{
    Serial.println("[DWDWX]: checking " + this->url);
    this->_http.begin(this->url);
    this->_http.setConnectTimeout(15e3);
    int httpCode = this->_http.GET();

    if (httpCode > 0)
    {
        this->dwdwxData = this->_http.getString();
        if (this->dwdwxData.length() == 0)
        {
            Serial.println("Error: Empty JSON response!");
            return true;
        }
        this->currentMessage = this->parseJSON(this->dwdwxData);
        Serial.println("[DWDWX]: Current Message = " + this->currentMessage);
        this->_dirty = true;
        this->_http.end();
        return true;
    }
    else
    {
        Serial.printf("HTTP request failed, error: %s\n", this->_http.errorToString(httpCode).c_str());
        this->_http.end();
        return false;
    }
}

char wx_messageBuffer[64];
String keyCurrent = "current";
String keyWX_Temperature = "temperature_2m";
String keyWX_Humidity = "relative_humidity_2m";
String keyWX_WindDir = "wind_direction_10m";
String keyWX_WindSpeed = "wind_speed_10m";
String keyWX_WindGusts = "wind_gusts_10m";
String keyWX_PressureMSL = "wind_gusts_10m";
String keyWX_CloudCover = "cloud_cover";

const char *wxFormat = "Temp %0.1f / rH %d / Wind %d deg  - %0.1f km/h, Boehe  %0.1fkm/h (gusts) / %0.1f hPa / Wolken: %d";
String wxRes;
String DWDWXClient::parseJSON(String jsonData)
{
    JsonDocument doc;
    deserializeJson(doc, jsonData);
    if (!doc[keyCurrent].isNull()) {
        sprintf(wx_messageBuffer, wxFormat,
            doc[keyCurrent][keyWX_Temperature].as<float>(), doc[keyCurrent][keyWX_Humidity].as<int>(),
            doc[keyCurrent][keyWX_WindDir].as<float>(), doc[keyCurrent][keyWX_WindSpeed].as<float>(),
            doc[keyCurrent][keyWX_WindGusts].as<float>(), doc[keyCurrent][keyWX_PressureMSL].as<float>(),
            doc[keyCurrent][keyWX_CloudCover].as<int>()
        );
        wxRes.clear();
        wxRes.concat(wx_messageBuffer);
        // delete []wx_messageBuffer;
        return wxRes;
    }
    return emptyString;
}

// #endif